//
// posix_event.hpp
// ~~~~~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef CPPCMS_BOOST_ASIO_DETAIL_POSIX_EVENT_HPP
#define CPPCMS_BOOST_ASIO_DETAIL_POSIX_EVENT_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <cppcms_boost/asio/detail/push_options.hpp>

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/config.hpp>
#include <cppcms_boost/system/system_error.hpp>
#include <cppcms_boost/asio/detail/pop_options.hpp>

#if defined(CPPCMS_BOOST_HAS_PTHREADS)

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/assert.hpp>
#include <cppcms_boost/throw_exception.hpp>
#include <pthread.h>
#include <cppcms_boost/asio/detail/pop_options.hpp>

#include <cppcms_boost/asio/error.hpp>
#include <cppcms_boost/asio/detail/noncopyable.hpp>

namespace cppcms_boost {
namespace asio {
namespace detail {

class posix_event
  : private noncopyable
{
public:
  // Constructor.
  posix_event()
    : signalled_(false)
  {
    int error = ::pthread_cond_init(&cond_, 0);
    if (error != 0)
    {
      cppcms_boost::system::system_error e(
          cppcms_boost::system::error_code(error,
            cppcms_boost::asio::error::get_system_category()),
          "event");
      cppcms_boost::throw_exception(e);
    }
  }

  // Destructor.
  ~posix_event()
  {
    ::pthread_cond_destroy(&cond_);
  }

  // Signal the event.
  template <typename Lock>
  void signal(Lock& lock)
  {
    CPPCMS_BOOST_ASSERT(lock.locked());
    (void)lock;
    signalled_ = true;
    ::pthread_cond_signal(&cond_); // Ignore EINVAL.
  }

  // Reset the event.
  template <typename Lock>
  void clear(Lock& lock)
  {
    CPPCMS_BOOST_ASSERT(lock.locked());
    (void)lock;
    signalled_ = false;
  }

  // Wait for the event to become signalled.
  template <typename Lock>
  void wait(Lock& lock)
  {
    CPPCMS_BOOST_ASSERT(lock.locked());
    while (!signalled_)
      ::pthread_cond_wait(&cond_, &lock.mutex().mutex_); // Ignore EINVAL.
  }

private:
  ::pthread_cond_t cond_;
  bool signalled_;
};

} // namespace detail
} // namespace asio
} // namespace boost

#endif // defined(BOOST_HAS_PTHREADS)

#include <cppcms_boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_DETAIL_POSIX_EVENT_HPP
