//
// tss_ptr.hpp
// ~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef CPPCMS_BOOST_ASIO_DETAIL_TSS_PTR_HPP
#define CPPCMS_BOOST_ASIO_DETAIL_TSS_PTR_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <cppcms_boost/asio/detail/push_options.hpp>

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/config.hpp>
#include <cppcms_boost/asio/detail/pop_options.hpp>

#if !defined(CPPCMS_BOOST_HAS_THREADS)
# include <cppcms_boost/asio/detail/null_tss_ptr.hpp>
#elif defined(CPPCMS_BOOST_WINDOWS)
# include <cppcms_boost/asio/detail/win_tss_ptr.hpp>
#elif defined(CPPCMS_BOOST_HAS_PTHREADS)
# include <cppcms_boost/asio/detail/posix_tss_ptr.hpp>
#else
# error Only Windows and POSIX are supported!
#endif

namespace cppcms_boost {
namespace asio {
namespace detail {

template <typename T>
class tss_ptr
#if !defined(CPPCMS_BOOST_HAS_THREADS)
  : public null_tss_ptr<T>
#elif defined(CPPCMS_BOOST_WINDOWS)
  : public win_tss_ptr<T>
#elif defined(CPPCMS_BOOST_HAS_PTHREADS)
  : public posix_tss_ptr<T>
#endif
{
public:
  void operator=(T* value)
  {
#if !defined(CPPCMS_BOOST_HAS_THREADS)
    null_tss_ptr<T>::operator=(value);
#elif defined(CPPCMS_BOOST_WINDOWS)
    win_tss_ptr<T>::operator=(value);
#elif defined(CPPCMS_BOOST_HAS_PTHREADS)
    posix_tss_ptr<T>::operator=(value);
#endif
  }
};

} // namespace detail
} // namespace asio
} // namespace boost

#include <cppcms_boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_DETAIL_TSS_PTR_HPP
