//
// wince_thread.hpp
// ~~~~~~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef CPPCMS_BOOST_ASIO_DETAIL_WINCE_THREAD_HPP
#define CPPCMS_BOOST_ASIO_DETAIL_WINCE_THREAD_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <cppcms_boost/asio/detail/push_options.hpp>

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/config.hpp>
#include <cppcms_boost/system/system_error.hpp>
#include <cppcms_boost/asio/detail/pop_options.hpp>

#if defined(CPPCMS_BOOST_WINDOWS) && defined(UNDER_CE)

#include <cppcms_boost/asio/error.hpp>
#include <cppcms_boost/asio/detail/noncopyable.hpp>
#include <cppcms_boost/asio/detail/socket_types.hpp>

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/throw_exception.hpp>
#include <memory>
#include <cppcms_boost/asio/detail/pop_options.hpp>

namespace cppcms_boost {
namespace asio {
namespace detail {

DWORD WINAPI wince_thread_function(LPVOID arg);

class wince_thread
  : private noncopyable
{
public:
  // Constructor.
  template <typename Function>
  wince_thread(Function f)
  {
    std::auto_ptr<func_base> arg(new func<Function>(f));
    DWORD thread_id = 0;
    thread_ = ::CreateThread(0, 0, wince_thread_function,
        arg.get(), 0, &thread_id);
    if (!thread_)
    {
      DWORD last_error = ::GetLastError();
      cppcms_boost::system::system_error e(
          cppcms_boost::system::error_code(last_error,
            cppcms_boost::asio::error::get_system_category()),
          "thread");
      cppcms_boost::throw_exception(e);
    }
    arg.release();
  }

  // Destructor.
  ~wince_thread()
  {
    ::CloseHandle(thread_);
  }

  // Wait for the thread to exit.
  void join()
  {
    ::WaitForSingleObject(thread_, INFINITE);
  }

private:
  friend DWORD WINAPI wince_thread_function(LPVOID arg);

  class func_base
  {
  public:
    virtual ~func_base() {}
    virtual void run() = 0;
  };

  template <typename Function>
  class func
    : public func_base
  {
  public:
    func(Function f)
      : f_(f)
    {
    }

    virtual void run()
    {
      f_();
    }

  private:
    Function f_;
  };

  ::HANDLE thread_;
};

inline DWORD WINAPI wince_thread_function(LPVOID arg)
{
  std::auto_ptr<wince_thread::func_base> func(
      static_cast<wince_thread::func_base*>(arg));
  func->run();
  return 0;
}

} // namespace detail
} // namespace asio
} // namespace boost

#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)

#include <cppcms_boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_DETAIL_WINCE_THREAD_HPP
