//
// placeholders.hpp
// ~~~~~~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef CPPCMS_BOOST_ASIO_PLACEHOLDERS_HPP
#define CPPCMS_BOOST_ASIO_PLACEHOLDERS_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <cppcms_boost/asio/detail/push_options.hpp>

#include <cppcms_boost/asio/detail/push_options.hpp>
#include <cppcms_boost/bind/arg.hpp>
#include <cppcms_boost/detail/workaround.hpp>
#include <cppcms_boost/asio/detail/pop_options.hpp>

namespace cppcms_boost {
namespace asio {
namespace placeholders {

#if defined(GENERATING_DOCUMENTATION)

/// An argument placeholder, for use with boost::bind(), that corresponds to
/// the error argument of a handler for any of the asynchronous functions.
unspecified error;

/// An argument placeholder, for use with boost::bind(), that corresponds to
/// the bytes_transferred argument of a handler for asynchronous functions such
/// as boost::asio::basic_stream_socket::async_write_some or
/// boost::asio::async_write.
unspecified bytes_transferred;

/// An argument placeholder, for use with boost::bind(), that corresponds to
/// the iterator argument of a handler for asynchronous functions such as
/// boost::asio::basic_resolver::resolve.
unspecified iterator;

#elif defined(__BORLANDC__) || defined(__GNUC__)

inline cppcms_boost::arg<1> error()
{
  return cppcms_boost::arg<1>();
}

inline cppcms_boost::arg<2> bytes_transferred()
{
  return cppcms_boost::arg<2>();
}

inline cppcms_boost::arg<2> iterator()
{
  return cppcms_boost::arg<2>();
}

#else

namespace detail
{
  template <int Number>
  struct placeholder
  {
    static cppcms_boost::arg<Number>& get()
    {
      static cppcms_boost::arg<Number> result;
      return result;
    }
  };
}

#if CPPCMS_BOOST_WORKAROUND(CPPCMS_BOOST_MSVC, < 1400)

static cppcms_boost::arg<1>& error
  = cppcms_boost::asio::placeholders::detail::placeholder<1>::get();
static cppcms_boost::arg<2>& bytes_transferred
  = cppcms_boost::asio::placeholders::detail::placeholder<2>::get();
static cppcms_boost::arg<2>& iterator
  = cppcms_boost::asio::placeholders::detail::placeholder<2>::get();

#else

namespace
{
  cppcms_boost::arg<1>& error
    = cppcms_boost::asio::placeholders::detail::placeholder<1>::get();
  cppcms_boost::arg<2>& bytes_transferred
    = cppcms_boost::asio::placeholders::detail::placeholder<2>::get();
  cppcms_boost::arg<2>& iterator
    = cppcms_boost::asio::placeholders::detail::placeholder<2>::get();
} // namespace

#endif

#endif

} // namespace placeholders
} // namespace asio
} // namespace boost

#include <cppcms_boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_PLACEHOLDERS_HPP
