/*
 *
 * Copyright (c) 1998-2002
 * John Maddock
 *
 * Use, modification and distribution are subject to the
 * Boost Software License, Version 1.0. (See accompanying file
 * LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 */

 /*
  *   LOCATION:    see http://www.boost.org for most recent version.
  *   FILE         instances.cpp
  *   VERSION      see <boost/version.hpp>
  *   DESCRIPTION: Defines those template instances that are placed in the
  *                library rather than in the users object files.
  */

//
// note no include guard, we may include this multiple times:
//
#ifndef CPPCMS_BOOST_REGEX_NO_EXTERNAL_TEMPLATES

namespace cppcms_boost{

//
// this header can be included multiple times, each time with
// a different character type, BOOST_REGEX_CHAR_T must be defined
// first:
//
#ifndef CPPCMS_BOOST_REGEX_CHAR_T
#  error "BOOST_REGEX_CHAR_T not defined"
#endif

#ifndef CPPCMS_BOOST_REGEX_TRAITS_T
#  define CPPCMS_BOOST_REGEX_TRAITS_T , cppcms_boost::regex_traits<CPPCMS_BOOST_REGEX_CHAR_T >
#endif

//
// what follows is compiler specific:
//

#if  defined(__BORLANDC__) && (__BORLANDC__ < 0x600)

#ifdef CPPCMS_BOOST_HAS_ABI_HEADERS
#  include CPPCMS_BOOST_ABI_PREFIX
#endif

#  ifndef CPPCMS_BOOST_REGEX_INSTANTIATE
#     pragma option push -Jgx
#  endif

template class CPPCMS_BOOST_REGEX_DECL basic_regex< CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >;
template class CPPCMS_BOOST_REGEX_DECL match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >;
#ifndef CPPCMS_BOOST_NO_STD_ALLOCATOR
template class CPPCMS_BOOST_REGEX_DECL ::cppcms_boost::re_detail::perl_matcher<CPPCMS_BOOST_REGEX_CHAR_T const *, match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >::allocator_type CPPCMS_BOOST_REGEX_TRAITS_T >;
#endif

#  ifndef CPPCMS_BOOST_REGEX_INSTANTIATE
#     pragma option pop
#  endif

#ifdef CPPCMS_BOOST_HAS_ABI_HEADERS
#  include CPPCMS_BOOST_ABI_SUFFIX
#endif

#elif defined(CPPCMS_BOOST_MSVC) || defined(__ICL)

#  ifndef CPPCMS_BOOST_REGEX_INSTANTIATE
#     ifdef __GNUC__
#        define template __extension__ extern template
#     else
#        if CPPCMS_BOOST_MSVC > 1310
#           define CPPCMS_BOOST_REGEX_TEMPLATE_DECL
#        endif
#        define template extern template
#     endif
#  endif

#ifndef CPPCMS_BOOST_REGEX_TEMPLATE_DECL
#  define CPPCMS_BOOST_REGEX_TEMPLATE_DECL CPPCMS_BOOST_REGEX_DECL
#endif

#  ifdef CPPCMS_BOOST_MSVC
#     pragma warning(push)
#     pragma warning(disable : 4251 4231 4660)
#  endif

template class CPPCMS_BOOST_REGEX_TEMPLATE_DECL basic_regex< CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >;

#if !CPPCMS_BOOST_WORKAROUND(CPPCMS_BOOST_MSVC, < 1300)
template class CPPCMS_BOOST_REGEX_TEMPLATE_DECL match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >;
#endif
#ifndef CPPCMS_BOOST_NO_STD_ALLOCATOR
template class CPPCMS_BOOST_REGEX_TEMPLATE_DECL ::cppcms_boost::re_detail::perl_matcher<CPPCMS_BOOST_REGEX_CHAR_T const *, match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >::allocator_type CPPCMS_BOOST_REGEX_TRAITS_T >;
#endif
#if !(defined(CPPCMS_BOOST_DINKUMWARE_STDLIB) && (CPPCMS_BOOST_DINKUMWARE_STDLIB <= 1))\
   && !(defined(CPPCMS_BOOST_INTEL_CXX_VERSION) && (CPPCMS_BOOST_INTEL_CXX_VERSION <= 800))\
   && !(defined(__SGI_STL_PORT) || defined(_STLPORT_VERSION))\
   && !defined(CPPCMS_BOOST_REGEX_ICU_INSTANCES)
#if !CPPCMS_BOOST_WORKAROUND(CPPCMS_BOOST_MSVC, < 1300)
template class CPPCMS_BOOST_REGEX_TEMPLATE_DECL match_results< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator >;
#endif
#ifndef CPPCMS_BOOST_NO_STD_ALLOCATOR
template class CPPCMS_BOOST_REGEX_TEMPLATE_DECL ::cppcms_boost::re_detail::perl_matcher< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator, match_results< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator >::allocator_type, cppcms_boost::regex_traits<CPPCMS_BOOST_REGEX_CHAR_T > >;
#endif
#endif


#  ifdef CPPCMS_BOOST_MSVC
#     pragma warning(pop)
#  endif

#  ifdef template
#     undef template
#  endif

#undef CPPCMS_BOOST_REGEX_TEMPLATE_DECL

#elif (defined(__GNUC__) && (__GNUC__ >= 3))

#  ifndef CPPCMS_BOOST_REGEX_INSTANTIATE
#     define template __extension__ extern template
#  endif

#if !defined(CPPCMS_BOOST_NO_STD_LOCALE) && !defined(CPPCMS_BOOST_REGEX_ICU_INSTANCES)
namespace re_detail{
template CPPCMS_BOOST_REGEX_DECL
std::locale cpp_regex_traits_base<CPPCMS_BOOST_REGEX_CHAR_T>::imbue(const std::locale& l);

template CPPCMS_BOOST_REGEX_DECL
cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::string_type 
   cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::transform_primary(const CPPCMS_BOOST_REGEX_CHAR_T* p1, const CPPCMS_BOOST_REGEX_CHAR_T* p2) const;
template CPPCMS_BOOST_REGEX_DECL
cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::string_type 
   cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::transform(const CPPCMS_BOOST_REGEX_CHAR_T* p1, const CPPCMS_BOOST_REGEX_CHAR_T* p2) const;
template CPPCMS_BOOST_REGEX_DECL
cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::string_type 
   cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::lookup_collatename(const CPPCMS_BOOST_REGEX_CHAR_T* p1, const CPPCMS_BOOST_REGEX_CHAR_T* p2) const;
template CPPCMS_BOOST_REGEX_DECL
void cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::init();
template CPPCMS_BOOST_REGEX_DECL
cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::char_class_type 
   cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::lookup_classname_imp(const CPPCMS_BOOST_REGEX_CHAR_T* p1, const CPPCMS_BOOST_REGEX_CHAR_T* p2) const;
#ifdef CPPCMS_BOOST_REGEX_BUGGY_CTYPE_FACET
template CPPCMS_BOOST_REGEX_DECL
bool cpp_regex_traits_implementation<CPPCMS_BOOST_REGEX_CHAR_T>::isctype(const CPPCMS_BOOST_REGEX_CHAR_T c, char_class_type mask) const;
#endif
} // namespace
template CPPCMS_BOOST_REGEX_DECL
int cpp_regex_traits<CPPCMS_BOOST_REGEX_CHAR_T>::toi(const CPPCMS_BOOST_REGEX_CHAR_T*& first, const CPPCMS_BOOST_REGEX_CHAR_T* last, int radix)const;
template CPPCMS_BOOST_REGEX_DECL
std::string cpp_regex_traits<CPPCMS_BOOST_REGEX_CHAR_T>::catalog_name(const std::string& name);
template CPPCMS_BOOST_REGEX_DECL
std::string& cpp_regex_traits<CPPCMS_BOOST_REGEX_CHAR_T>::get_catalog_name_inst();
template CPPCMS_BOOST_REGEX_DECL
std::string cpp_regex_traits<CPPCMS_BOOST_REGEX_CHAR_T>::get_catalog_name();
#ifdef CPPCMS_BOOST_HAS_THREADS
template CPPCMS_BOOST_REGEX_DECL
static_mutex& cpp_regex_traits<CPPCMS_BOOST_REGEX_CHAR_T>::get_mutex_inst();
#endif
#endif

template CPPCMS_BOOST_REGEX_DECL basic_regex<CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >& 
   basic_regex<CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >::do_assign(
      const CPPCMS_BOOST_REGEX_CHAR_T* p1, 
      const CPPCMS_BOOST_REGEX_CHAR_T* p2, 
      flag_type f);
template CPPCMS_BOOST_REGEX_DECL basic_regex<CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >::locale_type CPPCMS_BOOST_REGEX_CALL 
   basic_regex<CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >::imbue(locale_type l);

template CPPCMS_BOOST_REGEX_DECL void CPPCMS_BOOST_REGEX_CALL 
   match_results<const CPPCMS_BOOST_REGEX_CHAR_T*>::maybe_assign(
      const match_results<const CPPCMS_BOOST_REGEX_CHAR_T*>& m);

namespace re_detail{
template CPPCMS_BOOST_REGEX_DECL void perl_matcher<CPPCMS_BOOST_REGEX_CHAR_T const *, match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >::allocator_type CPPCMS_BOOST_REGEX_TRAITS_T >::construct_init(
      const basic_regex<CPPCMS_BOOST_REGEX_CHAR_T CPPCMS_BOOST_REGEX_TRAITS_T >& e, match_flag_type f);
template CPPCMS_BOOST_REGEX_DECL bool perl_matcher<CPPCMS_BOOST_REGEX_CHAR_T const *, match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >::allocator_type CPPCMS_BOOST_REGEX_TRAITS_T >::match();
template CPPCMS_BOOST_REGEX_DECL bool perl_matcher<CPPCMS_BOOST_REGEX_CHAR_T const *, match_results< const CPPCMS_BOOST_REGEX_CHAR_T* >::allocator_type CPPCMS_BOOST_REGEX_TRAITS_T >::find();
} // namespace

#if (defined(__GLIBCPP__) || defined(__GLIBCXX__)) \
   && !defined(CPPCMS_BOOST_REGEX_ICU_INSTANCES)\
   && !defined(__SGI_STL_PORT)\
   && !defined(_STLPORT_VERSION)
// std:basic_string<>::const_iterator instances as well:
template CPPCMS_BOOST_REGEX_DECL void CPPCMS_BOOST_REGEX_CALL 
   match_results<std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator>::maybe_assign(
      const match_results<std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator>& m);

namespace re_detail{
template CPPCMS_BOOST_REGEX_DECL void perl_matcher<std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator, match_results< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator >::allocator_type, cppcms_boost::regex_traits<CPPCMS_BOOST_REGEX_CHAR_T > >::construct_init(
      const basic_regex<CPPCMS_BOOST_REGEX_CHAR_T>& e, match_flag_type f);
template CPPCMS_BOOST_REGEX_DECL bool perl_matcher<std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator, match_results< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator >::allocator_type, cppcms_boost::regex_traits<CPPCMS_BOOST_REGEX_CHAR_T > >::match();
template CPPCMS_BOOST_REGEX_DECL bool perl_matcher<std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator, match_results< std::basic_string<CPPCMS_BOOST_REGEX_CHAR_T>::const_iterator >::allocator_type, cppcms_boost::regex_traits<CPPCMS_BOOST_REGEX_CHAR_T > >::find();
} // namespace
#endif

#  ifdef template
#     undef template
#  endif


#endif

} // namespace boost

#endif // BOOST_REGEX_NO_EXTERNAL_TEMPLATES





