
//  (C) Copyright John Maddock 2005.  
//  Use, modification and distribution are subject to the Boost Software License,
//  Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt).
//
//  See http://www.boost.org/libs/type_traits for most recent version including documentation.


#ifndef CPPCMS_BOOST_TT_EXTENT_HPP_INCLUDED
#define CPPCMS_BOOST_TT_EXTENT_HPP_INCLUDED

// should be the last #include
#include <cppcms_boost/type_traits/detail/size_t_trait_def.hpp>

namespace cppcms_boost {

namespace detail{

#if defined( __CODEGEARC__ )
    // wrap the impl as main trait provides additional MPL lambda support
    template < typename T, std::size_t N >
    struct extent_imp {
        static const std::size_t value = __array_extent(T, N);
    };

#else

template <class T, std::size_t N>
struct extent_imp
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = 0);
};
#if !defined(CPPCMS_BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION) && !defined(CPPCMS_BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS)
template <class T, std::size_t R, std::size_t N>
struct extent_imp<T[R], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};

template <class T, std::size_t R, std::size_t N>
struct extent_imp<T const[R], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};

template <class T, std::size_t R, std::size_t N>
struct extent_imp<T volatile[R], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};

template <class T, std::size_t R, std::size_t N>
struct extent_imp<T const volatile[R], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};

template <class T, std::size_t R>
struct extent_imp<T[R],0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = R);
};

template <class T, std::size_t R>
struct extent_imp<T const[R], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = R);
};

template <class T, std::size_t R>
struct extent_imp<T volatile[R], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = R);
};

template <class T, std::size_t R>
struct extent_imp<T const volatile[R], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = R);
};

#if !CPPCMS_BOOST_WORKAROUND(__BORLANDC__, < 0x600) && !defined(__IBMCPP__) &&  !CPPCMS_BOOST_WORKAROUND(__DMC__, CPPCMS_BOOST_TESTED_AT(0x840)) && !defined(__MWERKS__)
template <class T, std::size_t N>
struct extent_imp<T[], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};
template <class T, std::size_t N>
struct extent_imp<T const[], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};
template <class T, std::size_t N>
struct extent_imp<T volatile[], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};
template <class T, std::size_t N>
struct extent_imp<T const volatile[], N>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = (::cppcms_boost::detail::extent_imp<T, N-1>::value));
};
template <class T>
struct extent_imp<T[], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = 0);
};
template <class T>
struct extent_imp<T const[], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = 0);
};
template <class T>
struct extent_imp<T volatile[], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = 0);
};
template <class T>
struct extent_imp<T const volatile[], 0>
{
   CPPCMS_BOOST_STATIC_CONSTANT(std::size_t, value = 0);
};
#endif
#endif

#endif  // non-CodeGear implementation
}   // ::boost::detail

template <class T, std::size_t N = 0>
struct extent
   : public ::cppcms_boost::integral_constant<std::size_t, ::cppcms_boost::detail::extent_imp<T,N>::value>
{
#if CPPCMS_BOOST_WORKAROUND(CPPCMS_BOOST_MSVC, < 1300) 
   typedef ::cppcms_boost::integral_constant<std::size_t, ::cppcms_boost::detail::extent_imp<T,N>::value> base_; 
   using base_::value;
#endif
    CPPCMS_BOOST_MPL_AUX_LAMBDA_SUPPORT(1,extent,(T))
};

} // namespace boost

#include <cppcms_boost/type_traits/detail/size_t_trait_undef.hpp>

#endif // BOOST_TT_IS_MEMBER_FUNCTION_POINTER_HPP_INCLUDED
