//  (C) Copyright Daniel Frey and Robert Ramey 2009.
//  Use, modification and distribution are subject to the Boost Software License,
//  Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt).
//
//  See http://www.boost.org/libs/type_traits for most recent version including documentation.
 
#ifndef CPPCMS_BOOST_TT_IS_VIRTUAL_BASE_OF_HPP_INCLUDED
#define CPPCMS_BOOST_TT_IS_VIRTUAL_BASE_OF_HPP_INCLUDED

#include <cppcms_boost/type_traits/is_base_of.hpp>
#include <cppcms_boost/type_traits/is_same.hpp>
#include <cppcms_boost/mpl/and.hpp>
#include <cppcms_boost/mpl/not.hpp>

// should be the last #include
#include <cppcms_boost/type_traits/detail/bool_trait_def.hpp>

namespace cppcms_boost {
namespace detail {


#ifdef CPPCMS_BOOST_MSVC
#pragma warning( push )
#pragma warning( disable : 4584 )
#elif defined __GNUC__
#pragma GCC system_header
#endif

template<typename Base, typename Derived, typename tag>
struct is_virtual_base_of_impl
{
    CPPCMS_BOOST_STATIC_CONSTANT(bool, value = false);
};

template<typename Base, typename Derived>
struct is_virtual_base_of_impl<Base, Derived, mpl::true_>
{
#ifdef __BORLANDC__
    struct X : public virtual Derived, public virtual Base 
    {
       X();
       X(const X&);
       X& operator=(const X&);
       ~X()throw();
    };
    struct Y : public virtual Derived 
    {
       Y();
       Y(const Y&);
       Y& operator=(const Y&);
       ~Y()throw();
    };
#else
    struct X : Derived, virtual Base 
    {
       X();
       X(const X&);
       X& operator=(const X&);
       ~X()throw();
    };
    struct Y : Derived 
    {
       Y();
       Y(const Y&);
       Y& operator=(const Y&);
       ~Y()throw();
    };
#endif
    CPPCMS_BOOST_STATIC_CONSTANT(bool, value = (sizeof(X)==sizeof(Y)));
};

template<typename Base, typename Derived>
struct is_virtual_base_of_impl2
{
   typedef typename mpl::and_<is_base_of<Base, Derived>, mpl::not_<is_same<Base, Derived> > >::type tag_type;
   typedef is_virtual_base_of_impl<Base, Derived, tag_type> imp;
   CPPCMS_BOOST_STATIC_CONSTANT(bool, value = imp::value);
};

#ifdef CPPCMS_BOOST_MSVC
#pragma warning( pop )
#endif

} // namespace detail

CPPCMS_BOOST_TT_AUX_BOOL_TRAIT_DEF2(
      is_virtual_base_of
       , Base
       , Derived
       , (::cppcms_boost::detail::is_virtual_base_of_impl2<Base,Derived>::value) 
)

#ifndef CPPCMS_BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION
CPPCMS_BOOST_TT_AUX_BOOL_TRAIT_PARTIAL_SPEC2_2(typename Base,typename Derived,is_virtual_base_of,Base&,Derived,false)
CPPCMS_BOOST_TT_AUX_BOOL_TRAIT_PARTIAL_SPEC2_2(typename Base,typename Derived,is_virtual_base_of,Base,Derived&,false)
CPPCMS_BOOST_TT_AUX_BOOL_TRAIT_PARTIAL_SPEC2_2(typename Base,typename Derived,is_virtual_base_of,Base&,Derived&,false)
#endif

} // namespace boost

#include <cppcms_boost/type_traits/detail/bool_trait_undef.hpp>

#endif
